from django.apps import AppConfig as BaseAppConfig
from django.db.models.signals import post_migrate
from django.utils.translation import gettext_lazy as _


class SimpelPartnersConfig(BaseAppConfig):
    icon = "account-box-outline"
    default_auto_field = "django.db.models.BigAutoField"
    name = "simpel_partners"
    label = "simpel_partners"
    verbose_name = _("Partners")

    def ready(self):
        from . import signals  # NOQA

        post_migrate.connect(init_app, sender=self)
        return super().ready()


def init_app(**kwargs):
    pass


def init_demo_users():
    from simpel.simpel_auth.utils import create_demo_users

    usernames = {
        "partners_manager": "Partners Manager",
        "partners_admin": "Partners Admin",
    }
    create_demo_users(usernames)


def init_permissions():
    # from django.contrib.auth.models import Group
    # from django.db import transaction

    # from simpel.simpel_auth.utils import add_perms, get_perms_dict

    # from . import models

    # with transaction.atomic():
    #     partners_manager, _ = Group.objects.get_or_create(name="Partners Managers")
    #     partners_admin, _ = Group.objects.get_or_create(name="Partners Admin")

    #     actions = ["view", "add", "change", "delete"]
    #     imex = ["import", "export"]

    #     partner = get_perms_dict(actions + imex + ["change_partner_user", "activate", "verify"], models.Partner)

    #     view_groups = [
    #         partners_manager,
    #         partners_admin,
    #     ]
    #     change_groups = [
    #         partners_manager,
    #         partners_admin,
    #     ]
    #     add_groups = [
    #         partners_manager,
    #         partners_admin,
    #     ]
    #     delete_groups = [partners_admin]
    #     import_groups = [partners_admin]
    #     export_groups = [
    #         partners_manager,
    #         partners_admin,
    #     ]
    #     activate_groups = [
    #         partners_manager,
    #         partners_admin,
    #     ]
    #     verify_groups = [
    #         partners_manager,
    #         partners_admin,
    #     ]

    #     add_perms(activate_groups, [partner["activate"]])
    #     add_perms(verify_groups, [partner["verify"]])
    pass



from import_export.resources import ModelResource

from .models import Partner

partner_fields = [
    "id",
    "created_at",
    "reg_number",
    "inner_id",
    "partner_type",
    "user",
    "doc",
    "idn",
    "tax_id",
    "name",
    "text",
    "html",
    "is_active",
    "is_verified",
    "is_customer",
    "is_supplier",
]


class PartnerResource(ModelResource):
    class Meta:
        model = Partner
        fields = partner_fields
        export_order = partner_fields
        import_id_fields = ("inner_id",)



from .models import Partner


def get_partner_states():
    model_key = "%s.%s" % (Partner._meta.app_label, Partner._meta.model_name)
    states = dict()
    states["count_all"] = Partner.objects.all().count()
    states["count_active"] = Partner.objects.filter(is_active=True).count()
    states["count_verified"] = Partner.objects.filter(is_verified=True).count()
    return {model_key: states}



from django.contrib import admin, messages
from django.utils.text import Truncator
from django.utils.translation import gettext_lazy as _

from import_export.admin import ImportExportMixin

from .models import Partner, PartnerAddress, PartnerContact
from .resources import PartnerResource
from .settings import partners_settings


class PartnerAddressInline(admin.StackedInline):
    model = PartnerAddress
    extra = 0
    autocomplete_fields = ["country"]


class PartnerContactInline(admin.TabularInline):
    model = PartnerContact
    extra = 0


class PartnerAdmin(ImportExportMixin, admin.ModelAdmin):
    resource_class = PartnerResource
    list_display = ["inner_id", "partner_name", "user", "is_active"]
    search_fields = ["inner_id", "name"]
    date_hierarchy = "created_at"
    list_filter = ["partner_type", "is_active", "is_verified", "is_customer", "is_supplier"]
    inlines = [PartnerContactInline, PartnerAddressInline]
    actions = [
        "activate_partner",
        "verify_partner",
        "deactivate_partner",
        "deverify_partner",
    ]

    def has_view_permission(self, request, obj=None):
        return True

    def has_activate_permission(self, request, obj=None):
        return True

    def has_verify_permission(self, request, obj=None):
        return True

    def partner_name(self, obj):
        return Truncator(str(obj)).chars(60)

    def get_readonly_fields(self, request, obj=None):
        if not request.user.has_perms("change_partner_user"):
            return ["user"]
        return super().get_readonly_fields(request, obj)

    @admin.action(permissions=["activate"], description=_("Activate Partner"))
    def activate_partner(self, request, queryset):
        try:
            queryset.update(is_active=True)
            count = queryset.count()
            if count > 1:
                msg = _("Activate %s") % queryset.first()
            else:
                msg = _("Activate %s partners") % count
            messages.success(request, msg)
        except Exception as err:
            messages.error(request, err)

    @admin.action(permissions=["activate"], description=_("Deactivate Partner"))
    def deactivate_partner(self, request, queryset):
        try:
            queryset.update(is_active=False)
            count = queryset.count()
            if count > 1:
                msg = _("Deactivate %s") % queryset.first()
            else:
                msg = _("Deactivate %s partners") % count
            messages.success(request, msg)
        except Exception as err:
            messages.error(request, err)

    @admin.action(permissions=["verify"], description=_("Verify Partner"))
    def verify_partner(self, request, queryset):
        try:
            queryset.update(is_verified=True)
            count = queryset.count()
            if count > 1:
                msg = _("Verify %s") % queryset.first()
            else:
                msg = _("Verify %s partners") % count
            messages.success(request, msg)
        except Exception as err:
            messages.error(request, err)

    @admin.action(permissions=["verify"], description=_("Deverify Partner"))
    def deverify_partner(self, request, queryset):
        try:
            queryset.update(is_verified=False)
            count = queryset.count()
            if count > 1:
                msg = _("Deverify %s") % queryset.first()
            else:
                msg = _("Deverify %s partners") % count
            messages.success(request, msg)
        except Exception as err:
            messages.error(request, err)


admin.site.register(Partner, partners_settings.PARTNER_ADMIN)



from django.utils.translation import gettext_lazy as _
from drf_spectacular.utils import extend_schema, inline_serializer
from rest_framework import response, status
from rest_framework.decorators import action
from rest_framework.exceptions import MethodNotAllowed, PermissionDenied
from rest_framework.fields import BooleanField, CharField
from rest_framework.generics import get_object_or_404
from rest_framework.permissions import IsAdminUser, IsAuthenticated
from rest_framework.viewsets import ModelViewSet

from simpel.simpel_api.permissions import IsPartner

from ..models import Partner
from .serializers import PartnerSerializer

InlineConfirmSerializer = inline_serializer(name="ConfirmSerializer", fields={"confirm": BooleanField()})
InlineMessageSerializer = inline_serializer(name="MessageSerializer", fields={"message": CharField()})


class PartnerViewSet(ModelViewSet):
    serializer_class = PartnerSerializer
    permission_classes = [IsAuthenticated]

    # def get_permissions(self):
    #     if self.action in ["retrieve", "destroy", "update", "update_partials"]:
    #         return [IsAdminUser()]
    #     return super().get_permissions()

    # def get_queryset(self):
    #     qs = Partner.objects.all()
    #     return qs

    # def get_serializer_class(self):
    #     return super().get_serializer_class()

    # def get_instance(self):
    #     partner = get_object_or_404(Partner, user=self.request.user)
    #     return partner

    # @action(methods=["POST"], detail=False, serializer_class=PartnerSerializer)
    # def register(self, request, *args, **kwargs):
    #     serializer = self.get_serializer(data=request.data, context={"request", request})
    #     serializer.is_valid(raise_exception=True)
    #     self.perform_create(serializer)
    #     headers = self.get_success_headers(serializer.data)
    #     return response.Response(serializer.data, status=status.HTTP_201_CREATED, headers=headers)

    # @extend_schema(operation_id="partners_activate", request=InlineConfirmSerializer)
    # @action(
    #     methods=["POST"],
    #     detail=True,
    #     permission_classes=[IsAdminUser],
    #     url_path="activate",
    # )
    # def activate(self, *args, **kwargs):
    #     obj = self.get_object()
    #     obj.activate()
    #     serializer = self.get_serializer(instance=obj)
    #     return response.Response(serializer.data, status=status.HTTP_200_OK)

    # @extend_schema(operation_id="partners_deactivate", request=InlineConfirmSerializer)
    # @action(
    #     methods=["POST"],
    #     detail=True,
    #     permission_classes=[IsAdminUser],
    #     url_path="deactivate",
    # )
    # def deactivate(self, *args, **kwargs):
    #     obj = self.get_object()
    #     obj.deactivate()
    #     serializer = self.get_serializer(instance=obj)
    #     return response.Response(serializer.data, status=status.HTTP_200_OK)

    # @extend_schema(operation_id="partners_verify", request=InlineConfirmSerializer)
    # @action(
    #     methods=["POST"],
    #     permission_classes=[IsAdminUser],
    #     detail=True,
    #     url_path="verify",
    # )
    # def verify(self, request, *args, **kwargs):
    #     obj = self.get_object()
    #     obj.verify()
    #     serializer = self.get_serializer(instance=obj)
    #     return response.Response(serializer.data, status=status.HTTP_200_OK)

    # @action(methods=["GET", "PUT", "PATCH"], detail=False, serializer_class=PartnerSerializer)
    # def me(self, request, *args, **kwargs):
    #     self.get_object = self.get_instance
    #     if self.request.method == "GET":
    #         return self.retrieve(request, *args, **kwargs)
    #     elif self.request.method == "PUT":
    #         return self.update(request, *args, **kwargs)
    #     elif self.request.method == "PATCH":
    #         kwargs["is_partial"] = True
    #         return self.update(request, *args, **kwargs)
    #     else:
    #         raise MethodNotAllowed()

    # @extend_schema(operation_id="partners_me_deactivate")
    # @action(
    #     methods=["PUT"],
    #     detail=False,
    #     permission_classes=[IsPartner],
    #     serializer_class=PartnerSerializer,
    #     url_path="me/deactivate",
    # )
    # def me_deactivate(self, request, *args, **kwargs):
    #     obj = self.get_instance()
    #     obj.deactivate()
    #     serializer = self.get_serializer(instance=obj)
    #     return response.Response(serializer.data, status=status.HTTP_200_OK)

    # @action(
    #     methods=["GET", "POST"],
    #     detail=False,
    #     permission_classes=[IsPartner],
    #     url_path="me/address",
    # )
    # def me_address(self, request, *args, **kwargs):
    #     partner = self.request.user.partner
    #     if self.request.method == "GET":
    #         serializer = AddressSerializer(instance=partner.addresses.all(), many=True)
    #     if self.request.method == "POST":
    #         serializer = self.get_serializer(data=self.request.data)
    #         serializer.is_valid(raise_exception=True)
    #         serializer.save(partner=partner)
    #     return response.Response(data=serializer.data, status=status.HTTP_200_OK)

    # @extend_schema(operation_id="partners_me_address_set_primary", request=InlineConfirmSerializer)
    # @action(
    #     methods=["PUT"],
    #     detail=False,
    #     permission_classes=[IsPartner],
    #     serializer_class=AddressSerializer,
    #     url_path="me/address/(?P<id>[^/.]+)/primary",
    # )
    # def me_address_set_primary(self, request, id, *args, **kwargs):
    #     """Make selected address as primary"""
    #     partner = self.request.user.partner
    #     address = get_object_or_404(PartnerAddress, id=id, partner=partner)
    #     PartnerAddress.objects.update(primary=False)
    #     address.primary = True
    #     address.save()
    #     serializer = self.get_serializer(instance=address)
    #     return response.Response(serializer.data, status=status.HTTP_200_OK)

    # @action(
    #     methods=["PUT", "PATCH"],
    #     detail=False,
    #     permission_classes=[IsPartner],
    #     serializer_class=AddressSerializer,
    #     url_path="me/address/(?P<id>[^/.]+)",
    # )
    # def update_my_address(self, request, id, *args, **kwargs):
    #     """Update selected customer address"""
    #     partner = self.request.user.partner
    #     address = get_object_or_404(PartnerAddress, id=id, partner=partner)
    #     kwargs = {"instance": address, "data": request.data}
    #     if request.method == "PATCH":
    #         kwargs["is_partial"] = True
    #     serializer = self.get_serializer(**kwargs)
    #     serializer.is_valid(raise_exception=True)
    #     serializer.save(partner=partner)
    #     return response.Response(serializer.data, status=status.HTTP_200_OK)

    # @action(
    #     methods=["DELETE"],
    #     detail=False,
    #     permission_classes=[IsPartner],
    #     url_path="me/address/(?P<id>[^/.]+)",
    # )
    # def remove_my_address(self, request, id, *args, **kwargs):
    #     """Update selected customer address"""
    #     partner = self.request.user.partner
    #     address = get_object_or_404(PartnerAddress, id=id, partner=partner)
    #     if partner.address.objects.count() <= 1 or address.primary:
    #         raise PermissionDenied(_("Deleting primary address is not allowed"))
    #     address.delete()
    #     return response.Response({"message": _("Your address deleted")}, status=status.HTTP_200_OK)



from django.db import models


class PartnerAddressManager(models.Manager):
    def get_primary(self, partner):
        try:
            return self.get(partner=partner, primary=True)
        except self.model.DoesNotExist:
            return None



# Generated by Django 3.2.12 on 2022-03-24 19:48

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import phonenumber_field.modelfields
import simpel_utils.models.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('simpel_contacts', '0003_auto_20220324_1858'),
    ]

    operations = [
        migrations.CreateModel(
            name='Partner',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('reg_number', models.PositiveIntegerField(blank=True, editable=False, null=True, verbose_name='Reg number')),
                ('inner_id', models.CharField(blank=True, editable=False, max_length=50, null=True, unique=True, verbose_name='Inner ID')),
                ('created_at', models.DateTimeField(default=django.utils.timezone.now, editable=False, verbose_name='created at')),
                ('partner_type', models.CharField(choices=[('personal', 'Personal'), ('organization', 'Organization')], default='organization', max_length=255, verbose_name='Partner type')),
                ('doc', models.CharField(choices=[('KTP', 'Kartu Tanda Penduduk'), ('SIM', 'Surat Izin Mengemudi'), ('TDP', 'Tanda Daftar Perusahaan')], default='KTP', help_text='Identity documents', max_length=5, verbose_name='Identity Document')),
                ('idn', models.CharField(help_text='Identification Number', max_length=15, null=True, verbose_name='Identification Number')),
                ('tax_id', models.CharField(blank=True, help_text='Tax ID', max_length=15, null=True, verbose_name='Tax ID')),
                ('name', models.CharField(db_index=True, help_text='Can be person name/organization etc as needed.', max_length=255, verbose_name='name')),
                ('text', models.TextField(blank=True, default='No profile information', help_text='Describe your self/organization profile.', max_length=245, null=True)),
                ('html', models.TextField(blank=True, editable=False, max_length=2550, null=True)),
                ('attachment', models.FileField(blank=True, help_text='Partner file attachment: PDF format file.', null=True, upload_to='', verbose_name='Attachment')),
                ('is_active', models.BooleanField(default=False, editable=False)),
                ('is_verified', models.BooleanField(default=False, editable=False)),
                ('is_customer', models.BooleanField(default=True, verbose_name='Customer')),
                ('is_supplier', models.BooleanField(default=False, verbose_name='Supplier')),
                ('joined_at', models.DateTimeField(default=django.utils.timezone.now, editable=False)),
                ('modified_at', models.DateTimeField(default=django.utils.timezone.now, editable=False)),
                ('user', models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to=settings.AUTH_USER_MODEL, verbose_name='user')),
            ],
            options={
                'verbose_name': 'Partner',
                'verbose_name_plural': 'Partners',
                'db_table': 'simpel_partner',
                'ordering': ('name',),
                'permissions': (('import_partner', 'Can import Partner'), ('export_partner', 'Can export Partner'), ('change_partner_user', 'Can change Partner user account'), ('activate_partner', 'Can activate Partner'), ('verify_partner', 'Can verify Partner')),
            },
        ),
        migrations.CreateModel(
            name='PartnerContact',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('contact_type', models.CharField(choices=[('phone', 'Phone'), ('fax', 'Fax'), ('mobile', 'Mobile'), ('whatsapp', 'Whatsapp'), ('telegram', 'Telegram')], default='phone', help_text='E.g. Phone or mobile', max_length=255, verbose_name='type')),
                ('contact', phonenumber_field.modelfields.PhoneNumberField(blank=True, help_text='Contact number', max_length=128, region=None, verbose_name='Phone number')),
                ('is_verified', models.BooleanField(default=False, editable=False)),
                ('partner', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='contacts', to='simpel_partners.partner', verbose_name='Partner')),
            ],
            options={
                'verbose_name': 'Partner contact',
                'verbose_name_plural': 'Partner contacts',
            },
        ),
        migrations.CreateModel(
            name='PartnerAddress',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('primary', models.BooleanField(default=False)),
                ('address_type', models.CharField(choices=[('home', 'Home'), ('office', 'Office'), ('branch_office', 'Branch Office'), ('billing', 'Billing'), ('shipping', 'Shipping'), ('drop_shipping', 'Dropshipping'), ('deliverable', 'Deliverable'), ('else', 'Else')], default='billing', help_text='E.g. Shipping or billing', max_length=255, verbose_name='type')),
                ('title', models.CharField(blank=True, choices=[('Mr', 'Mr'), ('Miss', 'Miss'), ('Mrs', 'Mrs'), ('Ms', 'Ms'), ('Dr', 'Dr')], max_length=64, verbose_name='Title')),
                ('name', models.CharField(help_text='If blank, contact name will be used.', max_length=255, verbose_name='name')),
                ('line_1', models.CharField(max_length=255, null=True, verbose_name='line 1')),
                ('line_2', models.CharField(blank=True, max_length=255, null=True, verbose_name='line 2')),
                ('line_3', models.CharField(blank=True, max_length=255, null=True, verbose_name='line 3')),
                ('city', models.CharField(max_length=255, null=True, verbose_name='city')),
                ('province', models.CharField(max_length=255, null=True, verbose_name='province')),
                ('postcode', simpel_utils.models.fields.UppercaseCharField(blank=True, max_length=64, verbose_name='Post/Zip-code')),
                ('country', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='simpel_contacts.country', verbose_name='Country')),
                ('partner', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='addresses', to='simpel_partners.partner', verbose_name='Partner')),
            ],
            options={
                'verbose_name': 'Partner address',
                'verbose_name_plural': 'Partner addresses',
            },
        ),
    ]



from rest_framework import serializers

from ..models import Partner


class PartnerSerializer(serializers.ModelSerializer):
    class Meta:
        model = Partner
        fields = "__all__"
        read_only_fields = (
            "user",
            "active",
            "html",
            "verified",
            "is_customer",
            "is_supplier",
        )



"""
    This module is largely inspired by django-rest-framework settings.
    This module provides the `settings` object, that is used to access
    app settings, checking for user settings first, then falling
    back to the defaults.
"""
from typing import Any, Dict

from django.conf import settings
from django.test.signals import setting_changed
from django.utils.module_loading import import_string

SETTINGS_DOC = "https://gitlab.com/simpel-projects/simpel-partners"

SIMPEL_PARTNERS_DEFAULTS: Dict[str, Any] = {
    "PARTNER_ADMIN": "simpel_partners.admin.PartnerAdmin",
    "PARTNER_REGISTRATION_VIEW": "partners.views.PartnerRegistrationView",
}

# List of settings that may be in string import notation.
IMPORT_STRINGS = ["PARTNER_ADMIN"]

# List of settings that have been removed
REMOVED_SETTINGS = []


def perform_import(val, setting_name):
    """
    If the given setting is a string import notation,
    then perform the necessary import or imports.
    """
    if val is None:
        return None
    elif isinstance(val, str):
        return import_from_string(val, setting_name)
    elif isinstance(val, (list, tuple)):
        return [import_from_string(item, setting_name) for item in val]
    elif isinstance(val, dict):
        return {key: import_from_string(item, setting_name) for key, item in val.items()}
    return val


def import_from_string(val, setting_name):
    """
    Attempt to import a class from a string representation.
    """
    try:
        return import_string(val)
    except ImportError as e:
        msg = "Could not import '%s' for SIMPEL_PARTNERS setting '%s'. %s: %s." % (
            val,
            setting_name,
            e.__class__.__name__,
            e,
        )
        raise ImportError(msg)


class AppSettings:
    """
    This module is largely inspired by django-rest-framework settings.
    This module provides the `partners_settings` object, that is used to access
    app settings, checking for user settings first, then falling
    back to the defaults.
    """

    def __init__(self, user_settings=None, defaults=None, import_strings=None):
        if user_settings:
            self._user_settings = self.__check_user_settings(user_settings)
        self.defaults = defaults or SIMPEL_PARTNERS_DEFAULTS
        self.import_strings = import_strings or IMPORT_STRINGS
        self._cached_attrs = set()

    @property
    def user_settings(self):
        if not hasattr(self, "_user_settings"):
            self._user_settings = getattr(settings, "SIMPEL_PARTNERS", {})
        return self._user_settings

    def __getattr__(self, attr):
        if attr not in self.defaults:
            raise AttributeError("Invalid SIMPEL_PARTNERS settings: '%s'" % attr)

        try:
            # Check if present in user settings
            val = self.user_settings[attr]
        except KeyError:
            # Fall back to defaults
            val = self.defaults[attr]

        # Coerce import strings into classes
        if attr in self.import_strings:
            val = perform_import(val, attr)

        # Cache the result
        self._cached_attrs.add(attr)
        setattr(self, attr, val)
        return val

    def __check_user_settings(self, user_settings):
        for setting in REMOVED_SETTINGS:
            if setting in user_settings:
                raise RuntimeError(
                    "The '%s' setting has been removed. Please refer to '%s' for available settings."
                    % (setting, SETTINGS_DOC)
                )
        return user_settings

    def reload(self):
        for attr in self._cached_attrs:
            delattr(self, attr)
        self._cached_attrs.clear()
        if hasattr(self, "_user_settings"):
            delattr(self, "_user_settings")


partners_settings = AppSettings(None, SIMPEL_PARTNERS_DEFAULTS, IMPORT_STRINGS)


def reload_partners_settings(*args, **kwargs):
    setting = kwargs["setting"]
    if setting == "SIMPEL_PARTNERS":
        partners_settings.reload()


setting_changed.connect(reload_partners_settings)



import logging
import warnings

from django.contrib.auth import get_user_model
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.utils import timezone
from django.utils.functional import cached_property
from django.utils.translation import gettext_lazy as _

from simpel_contacts.abstracts import AbstractAddress, AbstractContact
from simpel_numerators.models import NumeratorMixin, NumeratorReset

from .managers import PartnerAddressManager

logger = logging.getLogger(__name__)

warnings.simplefilter("default")


class PartnerContact(AbstractContact):
    partner = models.ForeignKey(
        "simpel_partners.Partner",
        on_delete=models.CASCADE,
        related_name="contacts",
        verbose_name=_("Partner"),
    )

    class Meta:
        verbose_name = _("Partner contact")
        verbose_name_plural = _("Partner contacts")


class PartnerAddress(AbstractAddress):
    """
    A partner can have one or more addresses. This can be useful e.g. when
    determining tax which depends on the origin of the shipment.
    """

    partner = models.ForeignKey(
        "simpel_partners.Partner",
        on_delete=models.CASCADE,
        related_name="addresses",
        verbose_name=_("Partner"),
    )

    objects = PartnerAddressManager()

    class Meta:
        verbose_name = _("Partner address")
        verbose_name_plural = _("Partner addresses")

    def get_primary(self):
        return PartnerAddress.objects.get_primary(self.partner)

    def set_as_primary(self, conditional=False):
        old_primary = self.get_primary()
        if old_primary:
            if conditional:
                return False
            old_primary.primary = False
            old_primary.save()
        self.primary = True
        self.save()
        return True

    def save(self, *args, **kwargs):
        if self.get_primary() is None:
            self.primary = True
        super().save(*args, **kwargs)


class Partner(NumeratorMixin):
    PERSONAL = "personal"
    ORGANIZATION = "organization"
    PARTNER_TYPE = [
        (PERSONAL, _("Personal")),
        (ORGANIZATION, _("Organization")),
    ]
    KTP = "KTP"
    SIM = "SIM"
    TDP = "TDP"
    ID_DOCS = (
        (KTP, "Kartu Tanda Penduduk"),
        (SIM, "Surat Izin Mengemudi"),
        (TDP, "Tanda Daftar Perusahaan"),
    )
    user = models.OneToOneField(
        get_user_model(),
        on_delete=models.SET_NULL,
        verbose_name=_("user"),
        null=True,
        blank=True,
    )
    partner_type = models.CharField(
        _("Partner type"),
        max_length=255,
        choices=PARTNER_TYPE,
        default=ORGANIZATION,
    )
    doc = models.CharField(
        max_length=5,
        choices=ID_DOCS,
        default=KTP,
        verbose_name=_("Identity Document"),
        help_text="Identity documents",
    )
    idn = models.CharField(
        null=True,
        blank=False,
        max_length=15,
        verbose_name=_("Identification Number"),
        help_text=_("Identification Number"),
    )
    tax_id = models.CharField(
        null=True,
        blank=True,
        max_length=15,
        verbose_name=_("Tax ID"),
        help_text=_("Tax ID"),
    )
    name = models.CharField(
        _("name"),
        max_length=255,
        db_index=True,
        help_text=_("Can be person name/organization etc as needed."),
    )
    text = models.TextField(
        default="No profile information",
        null=True,
        max_length=245,
        blank=True,
        help_text=_("Describe your self/organization profile."),
    )
    # The html version of the user information.
    html = models.TextField(
        null=True,
        max_length=2550,
        blank=True,
        editable=False,
    )
    attachment = models.FileField(
        null=True, blank=True, verbose_name=_("Attachment"), help_text=_("Partner file attachment: PDF format file.")
    )

    is_active = models.BooleanField(
        default=False,
        editable=False,
    )
    is_verified = models.BooleanField(
        default=False,
        editable=False,
    )
    is_customer = models.BooleanField(
        default=True,
        verbose_name=_("Customer"),
    )
    is_supplier = models.BooleanField(
        default=False,
        verbose_name=_("Supplier"),
    )
    joined_at = models.DateTimeField(
        default=timezone.now,
        editable=False,
    )

    modified_at = models.DateTimeField(
        default=timezone.now,
        editable=False,
    )

    doc_prefix = "PRT"
    reset_mode = NumeratorReset.MONTHLY

    class Meta:
        db_table = "simpel_partner"
        verbose_name = _("Partner")
        verbose_name_plural = _("Partners")
        ordering = ("name",)
        permissions = (
            ("import_partner", _("Can import Partner")),
            ("export_partner", _("Can export Partner")),
            ("change_partner_user", _("Can change Partner user account")),
            ("activate_partner", _("Can activate Partner")),
            ("verify_partner", _("Can verify Partner")),
        )

    def __str__(self):
        return self.name

    def format_inner_id(self):
        """Inner ID final format"""
        form = [self.get_doc_prefix(), self.format_date(form="%m%y"), self.format_number()]
        inner_id = "{}.{}.{}".format(*form)
        return setattr(self, self.inner_id_field, inner_id)

    @classmethod
    def get_for_user(cls, user):
        partner, created = cls.objects.get_or_create(user=user, defaults={"name": user.get_full_name()})
        return partner

    def activate(self):
        if not self.is_active and not self.deleted:
            self.is_active = True
            self.save()
        else:
            raise PermissionError(_("Activation failed, make sure this customer is inactive and not deleted"))

    def deactivate(self):
        if self.is_active and not self.deleted:
            self.is_active = False
            self.save()
        else:
            raise PermissionError(_("This customer is active, or deleted"))

    def verify(self):
        if self.is_active and not (self.is_verified and self.deleted):
            self.is_verified = True
            self.save()
        else:
            raise PermissionError(_("This customer is inactive customer"))

    def get_address(self, address_type=None):
        address = None
        if address_type is not None:
            filters = {"address_type": address_type}
            address = self.addresses.filter(**filters).first()
        if address is None:
            address = address = self.addresses.get_primary(self)
        return address

    @cached_property
    def address(self):
        return self.addresses.get_primary(self)

    @cached_property
    def shipping_address(self):
        return self.get_address(address_type=PartnerAddress.SHIPPING)

    @cached_property
    def billing_address(self):
        return self.get_address(address_type=PartnerAddress.BILLING)

    @cached_property
    def deliverable_address(self):
        return self.get_address(address_type=PartnerAddress.DELIVERABLE)

    def get_deliverable_info(self):
        warnings.warn(
            "get_deliverable_info is deprecated, use partner.deleverable_address.to_dict instead",
            DeprecationWarning,
        )
        if self.deliverable_address is not None:
            return self.deliverable_address.to_dict()
        return dict()

    @cached_property
    def balance(self):
        # TODO Move this to real apps
        balance_account = self.get_balance_account()
        return 0.00 if balance_account is None else balance_account.balance

    def get_balance_account(self):
        # TODO Move this to real apps
        try:
            from simpel_journals.settings import simpel_accounts_settings as acc_settings

            ctype = ContentType.objects.get_for_model(self.__class__)
            balance_account = ctype.accounts.get(
                instance_id=self.id,
                account_type__name=acc_settings.NAMES["PARTNER_BALANCE"],
            )
            return balance_account
        except Exception:
            return None



# TODO  Maybe unused
def render_address(self, data):
    line1 = []
    if data.get("address", None):
        line1.append(data["address"])
    line2 = []
    if data.get("city", None):
        line2.append(data["city"])
    if data.get("province"):
        line2.append(data["province"])
    if data.get("country", None):
        line2.append(data["country"])
    if data.get("zipcode", None):
        line2.append(data["zipcode"])
    line1 = " ".join(line1)
    line2 = ", ".join(line2)
    return line1, line2


def is_customer(user):
    """
    Check user is valid customer (verified and active)
    """
    customer = getattr(user, "customer", None)
    if not customer:
        return False
    else:
        if not (customer.verified and customer.active):
            return False
        else:
            return True



from django.contrib.auth import REDIRECT_FIELD_NAME
from django.contrib.auth.decorators import user_passes_test


def login_required(
    function=None,
    redirect_field_name=REDIRECT_FIELD_NAME,
    login_url=None,
):
    """
    Decorator for views that checks that the user is logged in, redirecting
    to the log-in page if necessary.
    """
    actual_decorator = user_passes_test(
        lambda u: hasattr(u, "partner", False),
        login_url=login_url,
        redirect_field_name=redirect_field_name,
    )
    if function:
        return actual_decorator(function)
    return actual_decorator
